@@ -245,7 +245,10 @@ class bdist_esky(Command):
         #  Zip up the distribution
         print "zipping up the esky"
         zfname = os.path.join(self.dist_dir,"%s.%s.zip"%(fullname,platform,))
-        create_zipfile(self.bootstrap_dir,zfname,compress=True)
+        if hasattr(self.freezer_module,"zipit"):
+            self.freezer_module.zipit(self,self.bootstrap_dir,zfname)
+        else:
+            create_zipfile(self.bootstrap_dir,zfname,compress=True)
         shutil.rmtree(self.bootstrap_dir)
         
 
@@ -545,7 +548,7 @@ class bdist_esky_patch(Command):
             print "patching", target_esky, "against", source_esky, "=>", patchfile
             if not self.dry_run:
                 try:
-                    esky.patch.main(["-z","diff",source_esky,target_esky,patchfile])
+                    esky.patch.main(["-Z","diff",source_esky,target_esky,patchfile])
                 except:
                     import traceback
                     traceback.print_exc()
