@@ -101,6 +101,10 @@ def freeze(dist):
     bslib.writestr(zipfile.ZipInfo("esky/bootstrap.pyc",cdate),eskybscode)
     bslib.close()
     #  Copy any core dependencies
+    if "fcntl" not in sys.builtin_module_names:
+        for nm in os.listdir(dist.freeze_dir):
+            if nm.startswith("fcntl"):
+                dist.copy_to_bootstrap_env(nm)
     for nm in os.listdir(dist.freeze_dir):
         if is_core_dependency(nm):
             dist.copy_to_bootstrap_env(nm)
