@@ -24,7 +24,7 @@ from StringIO import StringIO
 from py2app.build_app import py2app, get_zipfile, Target
 
 import esky
-from esky.util import is_core_dependency
+from esky.util import is_core_dependency, create_zipfile
 
 
 def freeze(dist):
@@ -42,6 +42,8 @@ def freeze(dist):
     options["excludes"] = excludes
     # py2app can't simultaneously freeze multiple scripts.
     # We do a separate freeze of each then merge them together.
+    # The control info (name, icon, etc) for the app will be taken from
+    # the first script in the list.
     exes = list(dist.get_executables())
     if not exes:
         raise RuntimeError("no scripts specified")
@@ -55,14 +57,12 @@ def freeze(dist):
             _merge_dir(tempdir,dist.freeze_dir)
         finally:
             shutil.rmtree(tempdir)
-    #  Move the resulting app into the main freeze dir
-    for nm in os.listdir(dist.freeze_dir):
-        for nm2 in os.listdir(os.path.join(dist.freeze_dir,nm)):
-            os.rename(os.path.join(dist.freeze_dir,nm,nm2),
-                      os.path.join(dist.freeze_dir,nm2))
-            os.rmdir(os.path.join(dist.freeze_dir,nm))
-    #  Remove any .pyc files with a corresponding .py file
-    resdir = os.path.join(dist.freeze_dir,"Contents/Resources")
+    #  Remove any .pyc files with a corresponding .py file.
+    #  This helps avoid timestamp changes that might interfere with
+    #  the generation of useful patches between versions.
+    appnm = dist.distribution.get_name()+".app"
+    app_dir = os.path.join(dist.freeze_dir,appnm)
+    resdir = os.path.join(app_dir,"Contents/Resources")
     for (dirnm,_,filenms) in os.walk(resdir):
         for nm in filenms:
             if nm.endswith(".pyc"):
@@ -75,7 +75,7 @@ def freeze(dist):
                     os.unlink(pyfile+"o")
     #  Copy data files into the freeze dir
     for (src,dst) in dist.get_data_files():
-        dst = os.path.join(dist.freeze_dir,"Contents","Resources",dst)
+        dst = os.path.join(app_dir,"Contents","Resources",dst)
         dstdir = os.path.dirname(dst)
         if not os.path.isdir(dstdir):
             dist.mkpath(dstdir)
@@ -88,17 +88,30 @@ def freeze(dist):
     lib.close()
     #  Copy the core dependencies into the bootstrap env.
     pydir = "python%d.%d" % sys.version_info[:2]
-    dist.copy_to_bootstrap_env("Contents/Info.plist")
-    dist.copy_to_bootstrap_env("Contents/PkgInfo")
-    dist.copy_to_bootstrap_env("Contents/Frameworks/Python.framework")
-    dist.copy_to_bootstrap_env("Contents/Resources/include")
-    dist.copy_to_bootstrap_env("Contents/Resources/lib/"+pydir+"/config")
+    def copy_to_bootstrap_env(src,dst=None):
+        if dst is None:
+            dst = src
+        src = os.path.join(appnm,src)
+        dist.copy_to_bootstrap_env(src,dst)
+    copy_to_bootstrap_env("Contents/Info.plist")
+    copy_to_bootstrap_env("Contents/PkgInfo")
+    copy_to_bootstrap_env("Contents/Frameworks/Python.framework")
+    copy_to_bootstrap_env("Contents/Resources/include")
+    copy_to_bootstrap_env("Contents/Resources/lib/"+pydir+"/config")
     if "fcntl" not in sys.builtin_module_names:
         dynload = "Contents/Resources/lib/"+pydir+"/lib-dynload"
-        dist.copy_to_bootstrap_env(os.path.join(dynload,"fcntl.so"))
-    dist.copy_to_bootstrap_env("Contents/Resources/__error__.sh")
-    dist.copy_to_bootstrap_env("Contents/Resources/__boot__.py")
-    dist.copy_to_bootstrap_env("Contents/Resources/site.py")
+        for nm in os.listdir(os.path.join(app_dir,dynload)):
+            if nm.startswith("fcntl"):
+                copy_to_bootstrap_env(os.path.join(dynload,nm))
+    copy_to_bootstrap_env("Contents/Resources/__error__.sh")
+    copy_to_bootstrap_env("Contents/Resources/__boot__.py")
+    copy_to_bootstrap_env("Contents/Resources/site.py")
+    #  Copy any other mentioned files (icons etc) into the bootstrap env
+    with open(os.path.join(app_dir,"Contents","Info.plist"),"rt") as f:
+        infotxt = f .read()
+    for nm in os.listdir(os.path.join(app_dir,"Contents","Resources")):
+        if nm in infotxt:
+            copy_to_bootstrap_env("Contents/Resources/"+nm)
     #  Create the bootstraping code, using custom code if specified.
     #  It gets stored as plain python code in Contents/Resources/__boot__.py
     code_source = [inspect.getsource(esky.bootstrap)]
@@ -120,11 +133,23 @@ def freeze(dist):
         f.write("")
     # TODO: copy icons and other required resources
     #  Copy the loader program for each script into the bootstrap env.
-    dist.copy_to_bootstrap_env("Contents/MacOS/python")
+    copy_to_bootstrap_env("Contents/MacOS/python")
     for exe in dist.get_executables():
         if not exe.include_in_bootstrap_env:
             continue
-        exepath = dist.copy_to_bootstrap_env("Contents/MacOS/"+exe.name)
+        exepath = copy_to_bootstrap_env("Contents/MacOS/"+exe.name)
+
+
+def zipit(dist,bsdir,zfname):
+    """Create the final zipfile of the esky.
+
+    We customize this process for py2app, so that the zipfile contains a
+    toplevel "<appname>.app" directory.  This allows users to just extract
+    the zipfile and have a proper application all set up and working.
+    """
+    def get_arcname(fpath):
+        return os.path.join(dist.distribution.get_name()+".app",fpath)
+    return create_zipfile(bsdir,zfname,get_arcname,compress=True)
 
 
 def _make_py2app_cmd(dist_dir,distribution,options,exe):
