@@ -27,7 +27,7 @@ import esky
 import esky.patch
 import esky.sudo
 from esky import bdist_esky
-from esky.util import extract_zipfile, get_platform
+from esky.util import extract_zipfile, deep_extract_zipfile, get_platform
 from esky.fstransact import FSTransaction, files_differ
 
 try:
@@ -158,11 +158,11 @@ class TestEsky(unittest.TestCase):
         os.unlink(os.path.join(tdir,"dist","eskytester-0.3.%s.zip"%(platform,)))
         #  Check that the patches apply cleanly
         uzdir = os.path.join(tdir,"unzip")
-        extract_zipfile(os.path.join(tdir,"dist","eskytester-0.1.%s.zip"%(platform,)),uzdir)
+        deep_extract_zipfile(os.path.join(tdir,"dist","eskytester-0.1.%s.zip"%(platform,)),uzdir)
         with open(os.path.join(tdir,"dist","eskytester-0.3.%s.from-0.1.patch"%(platform,)),"rb") as f:
             esky.patch.apply_patch(uzdir,f)
         shutil.rmtree(uzdir)
-        extract_zipfile(os.path.join(tdir,"dist","eskytester-0.2.%s.zip"%(platform,)),uzdir)
+        deep_extract_zipfile(os.path.join(tdir,"dist","eskytester-0.2.%s.zip"%(platform,)),uzdir)
         with open(os.path.join(tdir,"dist","eskytester-0.3.%s.from-0.2.patch"%(platform,)),"rb") as f:
             esky.patch.apply_patch(uzdir,f)
         shutil.rmtree(uzdir)
@@ -178,9 +178,10 @@ class TestEsky(unittest.TestCase):
         extract_zipfile(zfname,deploydir)
         #  Run the scripts in order.
         if options["bdist_esky"]["freezer_module"] == "py2app":
-            cmd1 = os.path.join(deploydir,"Contents","MacOS","script1")
-            cmd2 = os.path.join(deploydir,"Contents","MacOS","script2")
-            cmd3 = os.path.join(deploydir,"Contents","MacOS","script3")
+            appdir = os.listdir(deploydir)[0]
+            cmd1 = os.path.join(deploydir,appdir,"Contents","MacOS","script1")
+            cmd2 = os.path.join(deploydir,appdir,"Contents","MacOS","script2")
+            cmd3 = os.path.join(deploydir,appdir,"Contents","MacOS","script3")
         else:
             if sys.platform == "win32":
                 cmd1 = os.path.join(deploydir,"script1.exe")
