@@ -40,10 +40,30 @@ def pairwise(iterable):
         pass
     return izip(a,b)
 
+
+def common_prefix(iterables):
+    """Find the longest common prefix of a series of iterables."""
+    iterables = iter(iterables)
+    try:
+        prefix = iterables.next()
+    except StopIteration:
+        raise ValueError("at least one iterable is required")
+    for item in iterables:
+        count = 0
+        for (c1,c2) in izip(prefix,item):
+            if c1 != c2:
+                break
+            count += 1
+        prefix = prefix[:count]
+    return prefix
+
 def appdir_from_executable(exepath):
     """Find the top-level application directory, given sys.executable."""
     vdir = _bs_appdir_from_executable(exepath)
-    return os.path.dirname(vdir)
+    appdir = os.path.dirname(vdir)
+    if os.path.exists(os.path.join(appdir,"esky-bootstrap.txt")):
+        appdir = os.path.dirname(appdir)
+    return appdir
 
 
 def extract_zipfile(source,target,name_filter=None):
@@ -65,7 +85,10 @@ def extract_zipfile(source,target,name_filter=None):
             if nm.endswith("/"):
                 continue
             if name_filter:
-                outfilenm = os.path.join(target,name_filter(nm))
+                outfilenm = name_filter(nm)
+                if outfilenm is None:
+                    continue
+                outfilenm = os.path.join(target,outfilenm)
             else:
                 outfilenm = os.path.join(target,nm)
             if not os.path.isdir(os.path.dirname(outfilenm)):
@@ -86,6 +109,39 @@ def extract_zipfile(source,target,name_filter=None):
         zf.close()
 
 
+def zipfile_common_prefix_dir(source):
+    """Find the common prefix directory of all files in a zipfile."""
+    zf = zipfile.ZipFile(source)
+    prefix = common_prefix(zf.namelist())
+    if "/" in prefix:
+        return prefix.rsplit("/",1)[0] + "/"
+    else:
+        return ""
+
+def deep_extract_zipfile(source,target,name_filter=None):
+    """Extract the deep contents of a zipfile into a target directory.
+
+    This is just like extract_zipfile() except that any common prefix dirs
+    are removed.  For example, if everything in the zipfile is under the
+    directory "example.app" then that prefix will be removed during unzipping.
+
+    This is useful to allow distribution of "friendly" zipfiles that don't
+    overwrite files in the current directory when extracted by hand.
+    """
+    prefix = zipfile_common_prefix_dir(source)
+    if prefix:
+        def new_name_filter(nm):
+            if not nm.startswith(prefix):
+                return None
+            if name_filter is not None:
+                return name_filter(nm[len(prefix):])
+            return nm[len(prefix):]
+    else:
+         new_name_filter = name_filter
+    return extract_zipfile(source,target,new_name_filter)
+
+
+
 def create_zipfile(source,target,get_zipinfo=None,members=None,compress=None):
     """Bundle the contents of a given directory into a zipfile.
 
@@ -94,7 +150,8 @@ def create_zipfile(source,target,get_zipinfo=None,members=None,compress=None):
 
     If given, the optional argument 'get_zipinfo' must be a function mapping
     filenames to ZipInfo objects.  It may also return None to indicate that
-    defaults should be used.
+    defaults should be used, or a string to indicate that defaults should be
+    used with a new archive name.
 
     If given, the optional argument 'members' must be an iterable yielding
     names or ZipInfo objects.  Files will be added to the archive in the
@@ -126,6 +183,8 @@ def create_zipfile(source,target,get_zipinfo=None,members=None,compress=None):
             fpath = os.path.join(source,fpath)
         if zinfo is None:
             zf.write(fpath,fpath[len(source)+1:])
+        elif isinstance(zinfo,basestring):
+            zf.write(fpath,zinfo)
         else:
             with open(fpath,"rb") as f:
                 zf.writestr(zinfo,f.read())
